@@ -30,6 +30,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <math.h>
+#include <errno.h>
 #include <cairo.h>
 
 #include <linux/input.h>
@@ -518,7 +519,8 @@ int main(int argc, char *argv[])
 
 	d = display_create(&argc, argv);
 	if (d == NULL) {
-		fprintf(stderr, "failed to create display: %m\n");
+		fprintf(stderr, "failed to create display: %s\n",
+			strerror(errno));
 		return -1;
 	}
 
