@@ -35,6 +35,7 @@
 #include <sys/mman.h>
 #include <signal.h>
 #include <time.h>
+#include <errno.h>
 
 #include <wayland-client.h>
 #include "shared/helpers.h"
@@ -141,14 +142,14 @@ create_shm_buffers(struct display *display, struct buffer **buffers,
 
 	fd = os_create_anonymous_file(size);
 	if (fd < 0) {
-		fprintf(stderr, "creating a buffer file for %d B failed: %m\n",
-			size);
+		fprintf(stderr, "creating a buffer file for %d B failed: %s\n",
+			size, strerror(errno));
 		return -1;
 	}
 
 	data = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
 	if (data == MAP_FAILED) {
-		fprintf(stderr, "mmap failed: %m\n");
+		fprintf(stderr, "mmap failed: %s\n", strerror(errno));
 		close(fd);
 		return -1;
 	}
@@ -480,7 +481,7 @@ window_emulate_rendering(struct window *window)
 
 	ret = nanosleep(&delay, NULL);
 	if (ret)
-		printf("nanosleep failed: %m\n");
+		printf("nanosleep failed: %s\n", strerror(errno));
 }
 
 static void
