@@ -32,6 +32,7 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <signal.h>
+#include <string.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <linux/vt.h>
@@ -129,14 +130,16 @@ setup_tty(struct launcher_direct *launcher, int tty)
 	if (tty == 0) {
 		launcher->tty = dup(tty);
 		if (launcher->tty == -1) {
-			weston_log("couldn't dup stdin: %m\n");
+			weston_log("couldn't dup stdin: %s\n",
+				   strerror(errno));
 			return -1;
 		}
 	} else {
 		snprintf(tty_device, sizeof tty_device, "/dev/tty%d", tty);
 		launcher->tty = open(tty_device, O_RDWR | O_CLOEXEC);
 		if (launcher->tty == -1) {
-			weston_log("couldn't open tty %s: %m\n", tty_device);
+			weston_log("couldn't open tty %s: %s\n", tty_device,
+				   strerror(errno));
 			return -1;
 		}
 	}
@@ -151,7 +154,7 @@ setup_tty(struct launcher_direct *launcher, int tty)
 
 	ret = ioctl(launcher->tty, KDGETMODE, &kd_mode);
 	if (ret) {
-		weston_log("failed to get VT mode: %m\n");
+		weston_log("failed to get VT mode: %s\n", strerror(errno));
 		return -1;
 	}
 	if (kd_mode != KD_TEXT) {
@@ -164,19 +167,22 @@ setup_tty(struct launcher_direct *launcher, int tty)
 	ioctl(launcher->tty, VT_WAITACTIVE, minor(buf.st_rdev));
 
 	if (ioctl(launcher->tty, KDGKBMODE, &launcher->kb_mode)) {
-		weston_log("failed to read keyboard mode: %m\n");
+		weston_log("failed to read keyboard mode: %s\n",
+			   strerror(errno));
 		goto err_close;
 	}
 
 	if (ioctl(launcher->tty, KDSKBMUTE, 1) &&
 	    ioctl(launcher->tty, KDSKBMODE, K_OFF)) {
-		weston_log("failed to set K_OFF keyboard mode: %m\n");
+		weston_log("failed to set K_OFF keyboard mode: %s\n",
+			   strerror(errno));
 		goto err_close;
 	}
 
 	ret = ioctl(launcher->tty, KDSETMODE, KD_GRAPHICS);
 	if (ret) {
-		weston_log("failed to set KD_GRAPHICS mode on tty: %m\n");
+		weston_log("failed to set KD_GRAPHICS mode on tty: %s\n",
+			   strerror(errno));
 		goto err_close;
 	}
 
@@ -255,10 +261,12 @@ launcher_direct_restore(struct weston_launcher *launcher_base)
 
 	if (ioctl(launcher->tty, KDSKBMUTE, 0) &&
 	    ioctl(launcher->tty, KDSKBMODE, launcher->kb_mode))
-		weston_log("failed to restore kb mode: %m\n");
+		weston_log("failed to restore kb mode: %s\n",
+			   strerror(errno));
 
 	if (ioctl(launcher->tty, KDSETMODE, KD_TEXT))
-		weston_log("failed to set KD_TEXT mode on tty: %m\n");
+		weston_log("failed to set KD_TEXT mode on tty: %s\n",
+			   strerror(errno));
 
 	/* We have to drop master before we switch the VT back in
 	 * VT_AUTO, so we don't risk switching to a VT with another
