@@ -225,7 +225,7 @@ launcher_logind_close(struct weston_launcher *launcher, int fd)
 	r = fstat(fd, &st);
 	close(fd);
 	if (r < 0) {
-		weston_log("logind: cannot fstat fd: %m\n");
+		weston_log("logind: cannot fstat fd: %s\n", strerror(errno));
 		return;
 	}
 
@@ -483,11 +483,10 @@ device_paused(struct launcher_logind *wl, DBusMessage *m)
 	 * "gone" means the device is gone. We handle it the same as "force" as
 	 * a following udev event will be caught, too.
 	 *
-	 * If it's our main DRM device and not "gone", tell the compositor to go asleep. */
+	 * If it's our main DRM device, tell the compositor to go asleep. */
+
 	if (!strcmp(type, "pause"))
 		launcher_logind_pause_device_complete(wl, major, minor);
-	else if (strcmp(type, "gone") == 0)
-		return;
 
 	if (wl->sync_drm && major == DRM_MAJOR)
 		launcher_logind_set_active(wl, false);
