@@ -101,7 +101,8 @@ bind_to_abstract_socket(int display)
 			     "%c/tmp/.X11-unix/X%d", 0, display);
 	size = offsetof(struct sockaddr_un, sun_path) + name_size;
 	if (bind(fd, (struct sockaddr *) &addr, size) < 0) {
-		weston_log("failed to bind to @%s: %m\n", addr.sun_path + 1);
+		weston_log("failed to bind to @%s: %s\n", addr.sun_path + 1,
+			   strerror(errno));
 		close(fd);
 		return -1;
 	}
@@ -131,7 +132,8 @@ bind_to_unix_socket(int display)
 	size = offsetof(struct sockaddr_un, sun_path) + name_size;
 	unlink(addr.sun_path);
 	if (bind(fd, (struct sockaddr *) &addr, size) < 0) {
-		weston_log("failed to bind to %s: %m\n", addr.sun_path);
+		weston_log("failed to bind to %s: %s\n", addr.sun_path,
+			   strerror(errno));
 		close(fd);
 		return -1;
 	}
@@ -229,6 +231,8 @@ weston_xserver_destroy(struct wl_listener *l, void *data)
 	if (wxs->loop)
 		weston_xserver_shutdown(wxs);
 
+	weston_debug_scope_destroy(wxs->wm_debug);
+
 	free(wxs);
 }
 
@@ -391,5 +395,10 @@ weston_module_init(struct weston_compositor *compositor)
 	wxs->destroy_listener.notify = weston_xserver_destroy;
 	wl_signal_add(&compositor->destroy_signal, &wxs->destroy_listener);
 
+	wxs->wm_debug = weston_compositor_add_debug_scope(wxs->compositor,
+			"xwm-wm-x11",
+			"XWM's window management X11 events\n",
+			NULL, NULL);
+
 	return 0;
 }
